@@ -21,6 +21,7 @@
 #include <linux/proc_fs.h>
 #include <linux/stat.h>
 #include <linux/init.h>
+#include <linux/spinlock.h>
 
 #include <asm/byteorder.h>
 
@@ -31,7 +32,6 @@
 #include <asm/sbus.h>
 #include <asm/dma.h>
 #include <asm/system.h>
-#include <asm/spinlock.h>
 #include <asm/machines.h>
 #include <asm/ptrace.h>
 #include <asm/pgtable.h>
@@ -223,9 +223,15 @@ static int qlogicpti_reset_hardware(struct Scsi_Host *host)
 	struct qlogicpti *qpti = (struct qlogicpti *) host->hostdata;
 	struct qlogicpti_regs *qregs = qpti->qregs;
 	u_short param[6];
+	unsigned short risc_code_addr;
 	int loop_count, i;
 	unsigned long flags;
 
+	if(qpti->is_pti != 0)
+		risc_code_addr = pti_risc_code_addr01;
+	else
+		risc_code_addr = sbus_risc_code_addr01;
+
 	save_flags(flags); cli();
 
 	qregs->hcctrl = HCCTRL_PAUSE;
@@ -264,7 +270,7 @@ static int qlogicpti_reset_hardware(struct Scsi_Host *host)
 
 	/* Get RISC to start executing the firmware code. */
 	param[0] = MBOX_EXEC_FIRMWARE;
-	param[1] = risc_code_addr01;
+	param[1] = risc_code_addr;
 	if(qlogicpti_mbox_command(qpti, param, 1)) {
 		printk(KERN_EMERG "qlogicpti%d: Cannot execute ISP firmware.\n",
 		       qpti->qpti_id);
@@ -347,26 +353,37 @@ static int qlogicpti_reset_hardware(struct Scsi_Host *host)
 
 #define PTI_RESET_LIMIT 400
 
-__initfunc(static int qlogicpti_load_firmware(struct qlogicpti *qpti))
+static int __init qlogicpti_load_firmware(struct qlogicpti *qpti)
 {
 	struct qlogicpti_regs *qregs = qpti->qregs;
 	unsigned short csum = 0;
 	unsigned short param[6];
+	unsigned short *risc_code, risc_code_addr, risc_code_length;
 	unsigned long flags;
 #if !defined(MODULE) && !defined(__sparc_v9__)
 	unsigned long dvma_addr;
 #endif
 	int i, timeout;
 
+	if(qpti->is_pti != 0) {
+		risc_code = &pti_risc_code01[0];
+		risc_code_addr = pti_risc_code_addr01;
+		risc_code_length = pti_risc_code_length01;
+	} else {
+		risc_code = &sbus_risc_code01[0];
+		risc_code_addr = sbus_risc_code_addr01;
+		risc_code_length = sbus_risc_code_length01;
+	}
+
 	save_flags(flags); cli();
 
 	/* Verify the checksum twice, one before loading it, and once
 	 * afterwards via the mailbox commands.
 	 */
-	for(i = 0; i < risc_code_length01; i++)
-		csum += risc_code01[i];
+	for(i = 0; i < risc_code_length; i++)
+		csum += risc_code[i];
 	if(csum) {
-		printk(KERN_EMERG "qlogicpti%d: AIeee, firmware checksum failed!",
+		printk(KERN_EMERG "qlogicpti%d: Aieee, firmware checksum failed!",
 		       qpti->qpti_id);
 		return 1;
 	}		
@@ -406,11 +423,6 @@ __initfunc(static int qlogicpti_load_firmware(struct qlogicpti *qpti))
 		qpti->differential = 0;
 	qregs->hcctrl = HCCTRL_REL;
 
-	/* XXX Talk to PTI engineer about the following, ISP always
-	 * XXX returns 0x4001 return status for stop firmware command,
-	 * XXX documentation claims this means the cmd is unsupported
-	 * XXX on this ISP.  I think something fishy is going on.
-	 */
 	param[0] = MBOX_STOP_FIRMWARE;
 	param[1] = param[2] = param[3] = param[4] = param[5] = 0;
 	if(qlogicpti_mbox_command(qpti, param, 1)) {
@@ -423,13 +435,13 @@ __initfunc(static int qlogicpti_load_firmware(struct qlogicpti *qpti))
 	/* Load the firmware. */
 #if !defined(MODULE) && !defined(__sparc_v9__)
 	if (sparc_cpu_model != sun4d) {
-		dvma_addr = (unsigned long) mmu_lockarea((char *)&risc_code01[0],
-							 (sizeof(u_short) * risc_code_length01));
+		dvma_addr = (unsigned long) mmu_lockarea((char *)&risc_code[0],
+							 (sizeof(u_short) * risc_code_length));
 		param[0] = MBOX_LOAD_RAM;
-		param[1] = risc_code_addr01;
+		param[1] = risc_code_addr;
 		param[2] = (dvma_addr >> 16);
 		param[3] = (dvma_addr & 0xffff);
-		param[4] = (sizeof(u_short) * risc_code_length01);
+		param[4] = (sizeof(u_short) * risc_code_length);
 		if(qlogicpti_mbox_command(qpti, param, 1) ||
 		   (param[0] != MBOX_COMMAND_COMPLETE)) {
 			printk(KERN_EMERG "qlogicpti%d: Firmware dload failed, I'm bolixed!\n",
@@ -437,14 +449,14 @@ __initfunc(static int qlogicpti_load_firmware(struct qlogicpti *qpti))
 			restore_flags(flags);
 			return 1;
 		}
-		mmu_unlockarea((char *)dvma_addr, (sizeof(u_short) * risc_code_length01));
+		mmu_unlockarea((char *)dvma_addr, (sizeof(u_short) * risc_code_length));
 	} else
 #endif
 	/* We need to do it this slow way always on Ultra, SS[12]000. */
-		for(i = 0; i < risc_code_length01; i++) {
+		for(i = 0; i < risc_code_length; i++) {
 			param[0] = MBOX_WRITE_RAM_WORD;
-			param[1] = risc_code_addr01 + i;
-			param[2] = risc_code01[i];
+			param[1] = risc_code_addr + i;
+			param[2] = risc_code[i];
 			if(qlogicpti_mbox_command(qpti, param, 1) ||
 			   param[0] != MBOX_COMMAND_COMPLETE) {
 				printk("qlogicpti%d: Firmware dload failed, I'm bolixed!\n",
@@ -464,7 +476,7 @@ __initfunc(static int qlogicpti_load_firmware(struct qlogicpti *qpti))
 
 	/* Ask ISP to verify the checksum of the new code. */
 	param[0] = MBOX_VERIFY_CHECKSUM;
-	param[1] = risc_code_addr01;
+	param[1] = risc_code_addr;
 	if(qlogicpti_mbox_command(qpti, param, 1) ||
 	   (param[0] != MBOX_COMMAND_COMPLETE)) {
 		printk(KERN_EMERG "qlogicpti%d: New firmware csum failure!\n",
@@ -475,7 +487,7 @@ __initfunc(static int qlogicpti_load_firmware(struct qlogicpti *qpti))
 
 	/* Start using newly downloaded firmware. */
 	param[0] = MBOX_EXEC_FIRMWARE;
-	param[1] = risc_code_addr01;
+	param[1] = risc_code_addr;
 	qlogicpti_mbox_command(qpti, param, 1);
 
 	param[0] = MBOX_ABOUT_FIRMWARE;
@@ -491,16 +503,18 @@ __initfunc(static int qlogicpti_load_firmware(struct qlogicpti *qpti))
 	qpti->fware_majrev = param[1];
 	qpti->fware_minrev = param[2];
 
-	/* Load scsi initiator ID and interrupt level into sbus static ram. */
-	param[0] = MBOX_WRITE_RAM_WORD;
-	param[1] = 0xff80;
-	param[2] = (unsigned short) qpti->scsi_id;
-	qlogicpti_mbox_command(qpti, param, 1);
+	if(qpti->is_pti != 0) {
+		/* Load scsi initiator ID and interrupt level into sbus static ram. */
+		param[0] = MBOX_WRITE_RAM_WORD;
+		param[1] = 0xff80;
+		param[2] = (unsigned short) qpti->scsi_id;
+		qlogicpti_mbox_command(qpti, param, 1);
 
-	param[0] = MBOX_WRITE_RAM_WORD;
-	param[1] = 0xff00;
-	param[2] = (unsigned short) 3;
-	qlogicpti_mbox_command(qpti, param, 1);
+		param[0] = MBOX_WRITE_RAM_WORD;
+		param[1] = 0xff00;
+		param[2] = (unsigned short) 3;
+		qlogicpti_mbox_command(qpti, param, 1);
+	}
 
 	restore_flags(flags);
 	return 0;
@@ -564,13 +578,13 @@ static inline void qlogicpti_set_hostdev_defaults(struct qlogicpti *qpti)
 }
 
 static void do_qlogicpti_intr_handler(int irq, void *dev_id, struct pt_regs *regs);
+#ifndef __sparc_v9__
+static void do_qlogicpti_intr_handler_sun4m(int irq, void *dev_id, struct pt_regs *regs);
+#endif
 
 /* Detect all PTI Qlogic ISP's in the machine. */
-__initfunc(int qlogicpti_detect(Scsi_Host_Template *tpnt))
+int __init qlogicpti_detect(Scsi_Host_Template *tpnt)
 {
-#ifdef __sparc_v9__
-	struct devid_cookie dcookie;
-#endif
 	struct qlogicpti *qpti, *qlink;
 	struct Scsi_Host *qpti_host;
 	struct linux_sbus *sbus;
@@ -584,13 +598,8 @@ __initfunc(int qlogicpti_detect(Scsi_Host_Template *tpnt))
 
 	tpnt->proc_dir = &proc_scsi_qlogicpti;
 	qptichain = 0;
-	if(!SBus_chain) {
-#ifdef __sparc_v9__
-		return 0; /* Could be a PCI-only machine. */
-#else
-		panic("No SBUS in qlogicpti_detect()");
-#endif
-	}
+	if(!SBus_chain)
+		return 0;
 	for_each_sbus(sbus) {
 		for_each_sbusdev(sbdev_iter, sbus) {
 			qpti_dev = sbdev_iter;
@@ -601,6 +610,16 @@ __initfunc(int qlogicpti_detect(Scsi_Host_Template *tpnt))
 			   strcmp(qpti_dev->prom_name, "QLGC,isp"))
 				continue;
 
+			/* Sometimes Antares cards come up not completely
+			 * setup, and we get a report of a zero IRQ.
+			 * Skip over them in such cases so we survive.
+			 */
+			if(qpti_dev->irqs[0] == 0) {
+				printk("qpti%d: Adapter reports no interrupt, "
+				       "skipping over this card.", nqptis);
+				continue;
+			}
+
 			/* Yep, register and allocate software state. */
 			qpti_host = scsi_register(tpnt, sizeof(struct qlogicpti));
 			if(!qpti_host)
@@ -612,6 +631,10 @@ __initfunc(int qlogicpti_detect(Scsi_Host_Template *tpnt))
 			/* We are wide capable, 16 targets. */
 			qpti_host->max_id = MAX_TARGETS;
 
+#ifdef __sparc_v9__
+			qpti_host->unchecked_isa_dma = 1;
+#endif
+
 			/* Setup back pointers and misc. state. */
 			qpti->qhost = qpti_host;
 			qpti->qdev = qpti_dev;
@@ -633,7 +656,8 @@ __initfunc(int qlogicpti_detect(Scsi_Host_Template *tpnt))
 				       sizeof(qpti->prom_name));
 			qpti->prom_node = qpti_node;
 			
-			is_pti = strcmp (qpti->prom_name, "QLGC,isp");
+			qpti->is_pti = is_pti =
+				(strcmp (qpti->prom_name, "QLGC,isp") != 0);
 
 			/* Setup the reg property for this device. */
 			prom_apply_sbus_ranges(qpti->qdev->my_bus,
@@ -667,42 +691,32 @@ __initfunc(int qlogicpti_detect(Scsi_Host_Template *tpnt))
 			qpti_host->n_io_port = (unsigned char)
 				qpti->qdev->reg_addrs[0].reg_size;
 
-			qpti_host->irq = qpti->irq = qpti->qdev->irqs[0].pri;
+			qpti_host->irq = qpti->irq = qpti->qdev->irqs[0];
 
-#ifndef __sparc_v9__
-			/* Allocate the irq only if necessary. */
+			/* On Ultra and S{S1,C2}000 we must always call request_irq for each
+			 * qpti, so that imap registers get setup etc.
+			 * But irq values are different in that case anyway...
+			 * Otherwise allocate the irq only if necessary.
+			 */
 			for_each_qlogicpti(qlink) {
 				if((qlink != qpti) && (qpti->irq == qlink->irq)) {
 					goto qpti_irq_acquired; /* BASIC rulez */
 				}
 			}
-			if(request_irq(qpti->qhost->irq, do_qlogicpti_intr_handler,
-				       SA_SHIRQ, "PTI Qlogic/ISP SCSI", NULL)) {
+			if(request_irq(qpti->qhost->irq, 
+#ifndef __sparc_v9__			
+				       (sparc_cpu_model == sun4m || sparc_cpu_model == sun4c) ?
+				           do_qlogicpti_intr_handler_sun4m :
+#endif
+				           do_qlogicpti_intr_handler,
+				       SA_SHIRQ, "PTI Qlogic/ISP SCSI", qpti)) {
 				printk("Cannot acquire PTI Qlogic/ISP irq line\n");
 				/* XXX Unmap regs, unregister scsi host, free things. */
 				continue;
 			}
 qpti_irq_acquired:
-			printk("qpti%d: IRQ %d ", qpti->qpti_id, qpti->qhost->irq);
-#else
-			/* On Ultra we must always call request_irq for each
-			 * qpti, so that imap registers get setup etc.
-			 */
-			dcookie.real_dev_id = qpti;
-			dcookie.imap = dcookie.iclr = 0;
-			dcookie.pil = -1;
-			dcookie.bus_cookie = sbus;
-			if(request_irq(qpti->qhost->irq, do_qlogicpti_intr_handler,
-				       (SA_SHIRQ | SA_SBUS | SA_DCOOKIE),
-				       "PTI Qlogic/ISP SCSI", &dcookie)) {
-				printk("Cannot acquire PTI Qlogic/ISP irq line\n");
-				/* XXX Unmap regs, unregister scsi host, free things. */
-				continue;
-			}
-			qpti->qhost->irq = qpti->irq = dcookie.ret_ino;
-			printk("qpti%d: INO[%x] IRQ %d ",
-			       qpti->qpti_id, qpti->qhost->irq, dcookie.ret_pil);
-#endif
+			printk("qpti%d: IRQ %s ",
+			       qpti->qpti_id, __irq_itoa(qpti->qhost->irq));
 
 			/* Figure out our scsi ID on the bus */
 			qpti->scsi_id = prom_getintdefault(qpti->prom_node,
@@ -733,6 +747,9 @@ qpti_irq_acquired:
 				bsizes = (DMA_BURST32 - 1);
 			qpti->bursts = bsizes;
 
+			/* Clear out Scsi_Cmnd array. */
+			memset(qpti->cmd_slots, 0, sizeof(qpti->cmd_slots));
+
 			/* The request and response queues must each be aligned
 			 * on a page boundry.
 			 */
@@ -752,11 +769,13 @@ qpti_irq_acquired:
 			/* Set adapter and per-device default values. */
 			qlogicpti_set_hostdev_defaults(qpti);
 			
-			if (is_pti) {
-				/* Load the firmware. */
-				if(qlogicpti_load_firmware(qpti))
-					panic("PTI Qlogic/ISP firmware load failed");
+			/* Load the firmware. */
+			/* XXX Find out where is it possible to download
+			   our sbus_risc_code on non-PTI ISP1000. */
+			if(is_pti && qlogicpti_load_firmware(qpti))
+				panic("SBUS Qlogic/ISP firmware load failed");
 
+			if (is_pti) {
 				/* Check the PTI status reg. */
 				if(qlogicpti_verify_tmon(qpti))
 					panic("PTI Qlogic/ISP tmon verification failed");
@@ -773,7 +792,10 @@ qpti_irq_acquired:
 				char buffer[60];
 				
 				prom_getstring (qpti_node, "isp-fcode", buffer, 60);
-				printk("(Firmware %s)", buffer);
+				if (buffer[0])
+					printk("(Firmware %s)", buffer);
+				if (prom_getbool(qpti_node, "differential"))
+					qpti->differential = 1;
 			}
 			
 			printk (" [%s Wide, using %s interface]\n",
@@ -813,8 +835,8 @@ const char *qlogicpti_info(struct Scsi_Host *host)
 	static char buf[80];
 	struct qlogicpti *qpti = (struct qlogicpti *) host->hostdata;
 
-	sprintf(buf, "PTI Qlogic,ISP SBUS SCSI irq %d regs at %08lx",
-		host->irq, (unsigned long) qpti->qregs);
+	sprintf(buf, "PTI Qlogic,ISP SBUS SCSI irq %s regs at %08lx",
+		__irq_itoa(qpti->qhost->irq), (unsigned long) qpti->qregs);
 	return buf;
 }
 
@@ -836,11 +858,6 @@ static inline void cmd_frob(struct Command_Entry *cmd, Scsi_Cmnd *Cmnd,
 	memset(cmd, 0, sizeof(struct Command_Entry));
 	cmd->hdr.entry_cnt = 1;
 	cmd->hdr.entry_type = ENTRY_COMMAND;
-#ifdef __sparc_v9__
-	cmd->handle = (u_int) (((unsigned long)Cmnd) - PAGE_OFFSET); /* magic mushroom */
-#else
-	cmd->handle = (u_int) ((unsigned long)Cmnd);                 /* magic mushroom */
-#endif
 	cmd->target_id = Cmnd->target;
 	cmd->target_lun = Cmnd->lun;
 	cmd->cdb_length = Cmnd->cmd_len;
@@ -929,9 +946,15 @@ static inline u_int load_cmd(Scsi_Cmnd *Cmnd, struct Command_Entry *cmd,
 		cmd->dataseg[0].d_count = Cmnd->request_bufflen;
 		cmd->segment_cnt = 1;
 	}
+
+	/* Committed, record Scsi_Cmd so we can find it later. */
+	cmd->handle = in_ptr;
+	qpti->cmd_slots[in_ptr] = Cmnd;
+
 	qpti->cmd_count[Cmnd->target]++;
 	qregs->mbox4 = in_ptr;
 	qpti->req_in_ptr = in_ptr;
+
 	return in_ptr;
 }
 
@@ -964,7 +987,14 @@ int qlogicpti_queuecommand(Scsi_Cmnd *Cmnd, void (*done)(Scsi_Cmnd *))
 	Cmnd->scsi_done = done;
 	in_ptr = NEXT_REQ_PTR(in_ptr);
 	if(in_ptr == out_ptr) {
-		printk(KERN_EMERG "qlogicpti%d: request queue overflow\n", qpti->qpti_id);
+		printk(KERN_EMERG "qlogicpti%d: request queue overflow\n",qpti->qpti_id);
+
+		/* Unfortunately, unless you use the new EH code, which
+		 * we don't, the midlayer will ignore the return value,
+		 * which is insane.  We pick up the pieces like this.
+		 */
+		Cmnd->result = DID_BUS_BUSY;
+		done(Cmnd);
 		return 1;
 	}
 	if(qpti->send_marker) {
@@ -973,15 +1003,30 @@ int qlogicpti_queuecommand(Scsi_Cmnd *Cmnd, void (*done)(Scsi_Cmnd *))
 		if(NEXT_REQ_PTR(in_ptr) == out_ptr) {
 			qregs->mbox4 = in_ptr;
 			qpti->req_in_ptr = in_ptr;
-			printk(KERN_EMERG "qlogicpti%d: request queue overflow\n", qpti->qpti_id);
+			printk(KERN_EMERG "qlogicpti%d: request queue overflow\n",
+			       qpti->qpti_id);
+
+			/* Unfortunately, unless you use the new EH code, which
+			 * we don't, the midlayer will ignore the return value,
+			 * which is insane.  We pick up the pieces like this.
+			 */
+			Cmnd->result = DID_BUS_BUSY;
+			done(Cmnd);
 			return 1;
 		}
 		cmd = (struct Command_Entry *) &qpti->req_cpu[in_ptr];
 		in_ptr = NEXT_REQ_PTR(in_ptr);
 	}
 	cmd_frob(cmd, Cmnd, qpti);
-	if((in_ptr = load_cmd(Cmnd, cmd, qpti, qregs, in_ptr, out_ptr)) == -1)
+	if((in_ptr = load_cmd(Cmnd, cmd, qpti, qregs, in_ptr, out_ptr)) == -1) {
+		/* Unfortunately, unless you use the new EH code, which
+		 * we don't, the midlayer will ignore the return value,
+		 * which is insane.  We pick up the pieces like this.
+		 */
+		Cmnd->result = DID_BUS_BUSY;
+		done(Cmnd);
 		return 1;
+	}
 	update_can_queue(host, in_ptr, out_ptr);
 	return 0;
 }
@@ -1050,162 +1095,98 @@ static int qlogicpti_return_status(struct Status_Entry *sts)
 	return (sts->scsi_status & STATUS_MASK) | (host_status << 16);
 }
 
-#ifndef __sparc_v9__
-
-static __inline__ void qlogicpti_intr_handler(int irq, void *dev_id,
-					      struct pt_regs *regs)
+static __inline__ int qlogicpti_intr_handler(struct qlogicpti *qpti)
 {
-	static int running = 0;
 	Scsi_Cmnd *Cmnd;
 	struct Status_Entry *sts;
-	struct qlogicpti *qpti;
 	u_int in_ptr, out_ptr;
-	int again;
+	struct qlogicpti_regs *qregs;
 
-	/* It is ok to take irq's on one qpti while the other
-	 * is amidst the processing of a reset.
-	 */
-	running++;
+	if(!(qpti->qregs->sbus_stat & SBUS_STAT_RINT))
+		return 0;
+		
+	qregs = qpti->qregs;
 
-#if 0 /* XXX Investigate why resets cause this with one controller. */
-	if(running > qptis_running)
-		printk("qlogicpti_intr_handler: yieee, recursive interrupt!\n");
-#endif
-
-	/* Handle all ISP interrupts showing */
-repeat:
-	again = 0;
-	for_each_qlogicpti(qpti) {
-		if(qpti->qregs->sbus_stat & SBUS_STAT_RINT) {
-			struct qlogicpti_regs *qregs = qpti->qregs;
-
-			again = 1;
-			in_ptr = qregs->mbox5;
-			qregs->hcctrl = HCCTRL_CRIRQ;
-			if(qregs->sbus_semaphore & SBUS_SEMAPHORE_LCK) {
-				switch(qregs->mbox0) {
-				case ASYNC_SCSI_BUS_RESET:
-				case EXECUTION_TIMEOUT_RESET:
-					qpti->send_marker = 1;
-					break;
-				case INVALID_COMMAND:
-				case HOST_INTERFACE_ERROR:
-				case COMMAND_ERROR:
-				case COMMAND_PARAM_ERROR:
-					break;
-				}
-				qregs->sbus_semaphore = 0;
-			}
-
-			/* This looks like a network driver! */
-			out_ptr = qpti->res_out_ptr;
-			while(out_ptr != in_ptr) {
-				sts = (struct Status_Entry *) &qpti->res_cpu[out_ptr];
-				out_ptr = NEXT_RES_PTR(out_ptr);
-				Cmnd = (Scsi_Cmnd *) ((unsigned long)sts->handle);
-				if(sts->completion_status == CS_RESET_OCCURRED ||
-				   sts->completion_status == CS_ABORTED ||
-				   (sts->status_flags & STF_BUS_RESET))
-					qpti->send_marker = 1;
-
-				if(sts->state_flags & SF_GOT_SENSE)
-					memcpy(Cmnd->sense_buffer, sts->req_sense_data,
-					       sizeof(Cmnd->sense_buffer));
-
-				if(sts->hdr.entry_type == ENTRY_STATUS)
-					Cmnd->result = qlogicpti_return_status(sts);
-				else
-					Cmnd->result = DID_ERROR << 16;
-
-				if(Cmnd->use_sg)
-					mmu_release_scsi_sgl((struct mmu_sglist *)
-							     Cmnd->buffer,
-							     Cmnd->use_sg - 1,
-							     qpti->qdev->my_bus);
-				else
-					mmu_release_scsi_one((__u32)Cmnd->SCp.ptr,
-							     Cmnd->request_bufflen,
-							     qpti->qdev->my_bus);
-
-				qpti->cmd_count[Cmnd->target]--;
-				qregs->mbox5 = out_ptr;
-				Cmnd->scsi_done(Cmnd);
-			}
-			qpti->res_out_ptr = out_ptr;
+	in_ptr = qregs->mbox5;
+	qregs->hcctrl = HCCTRL_CRIRQ;
+	if(qregs->sbus_semaphore & SBUS_SEMAPHORE_LCK) {
+		switch(qregs->mbox0) {
+		case ASYNC_SCSI_BUS_RESET:
+		case EXECUTION_TIMEOUT_RESET:
+			qpti->send_marker = 1;
+			break;
+		case INVALID_COMMAND:
+		case HOST_INTERFACE_ERROR:
+		case COMMAND_ERROR:
+		case COMMAND_PARAM_ERROR:
+			break;
 		}
+		qregs->sbus_semaphore = 0;
 	}
-	if(again)
-		goto repeat;
-	running--;
+
+	/* This looks like a network driver! */
+	out_ptr = qpti->res_out_ptr;
+	while(out_ptr != in_ptr) {
+		u_int cmd_slot;
+
+		sts = (struct Status_Entry *) &qpti->res_cpu[out_ptr];
+		out_ptr = NEXT_RES_PTR(out_ptr);
+
+		/* We store an index in the handle, not the pointer in
+		 * some form.  This avoids problems due to the fact
+		 * that the handle provided is only 32-bits. -DaveM
+		 */
+		cmd_slot = sts->handle;
+		Cmnd = qpti->cmd_slots[cmd_slot];
+		qpti->cmd_slots[cmd_slot] = NULL;
+
+		if(sts->completion_status == CS_RESET_OCCURRED ||
+		   sts->completion_status == CS_ABORTED ||
+		   (sts->status_flags & STF_BUS_RESET))
+			qpti->send_marker = 1;
+
+		if(sts->state_flags & SF_GOT_SENSE)
+			memcpy(Cmnd->sense_buffer, sts->req_sense_data,
+			       sizeof(Cmnd->sense_buffer));
+
+		if(sts->hdr.entry_type == ENTRY_STATUS)
+			Cmnd->result = qlogicpti_return_status(sts);
+		else
+			Cmnd->result = DID_ERROR << 16;
+
+		if(Cmnd->use_sg)
+			mmu_release_scsi_sgl((struct mmu_sglist *)
+					     Cmnd->buffer,
+					     Cmnd->use_sg - 1,
+					     qpti->qdev->my_bus);
+		else
+			mmu_release_scsi_one((__u32)((unsigned long)Cmnd->SCp.ptr),
+					     Cmnd->request_bufflen,
+					     qpti->qdev->my_bus);
+
+		qpti->cmd_count[Cmnd->target]--;
+		qregs->mbox5 = out_ptr;
+		Cmnd->scsi_done(Cmnd);
+	}
+	qpti->res_out_ptr = out_ptr;
+	return 1;
 }
 
-#else /* __sparc_v9__ */
+#ifndef __sparc_v9__
 
-static __inline__ void qlogicpti_intr_handler(int irq, void *dev_id,
-					      struct pt_regs *regs)
+static void do_qlogicpti_intr_handler_sun4m(int irq, void *dev_id, struct pt_regs *regs)
 {
-	struct qlogicpti *qpti = dev_id;
-	Scsi_Cmnd *Cmnd;
-	struct Status_Entry *sts;
-	u_int in_ptr, out_ptr;
+	unsigned long flags;
+	struct qlogicpti *qpti;
+	int again;
 
-	if(qpti->qregs->sbus_stat & SBUS_STAT_RINT) {
-		struct qlogicpti_regs *qregs = qpti->qregs;
-
-		in_ptr = qregs->mbox5;
-		qregs->hcctrl = HCCTRL_CRIRQ;
-		if(qregs->sbus_semaphore & SBUS_SEMAPHORE_LCK) {
-			switch(qregs->mbox0) {
-			case ASYNC_SCSI_BUS_RESET:
-			case EXECUTION_TIMEOUT_RESET:
-				qpti->send_marker = 1;
-				break;
-			case INVALID_COMMAND:
-			case HOST_INTERFACE_ERROR:
-			case COMMAND_ERROR:
-			case COMMAND_PARAM_ERROR:
-				break;
-			}
-			qregs->sbus_semaphore = 0;
-		}
-
-		/* This looks like a network driver! */
-		out_ptr = qpti->res_out_ptr;
-		while(out_ptr != in_ptr) {
-			sts = (struct Status_Entry *) &qpti->res_cpu[out_ptr];
-			out_ptr = NEXT_RES_PTR(out_ptr);
-			Cmnd = (Scsi_Cmnd *) (((unsigned long)sts->handle)+PAGE_OFFSET);
-
-			if(sts->completion_status == CS_RESET_OCCURRED ||
-			   sts->completion_status == CS_ABORTED ||
-			   (sts->status_flags & STF_BUS_RESET))
-				qpti->send_marker = 1;
-
-			if(sts->state_flags & SF_GOT_SENSE)
-				memcpy(Cmnd->sense_buffer, sts->req_sense_data,
-				       sizeof(Cmnd->sense_buffer));
-
-			if(sts->hdr.entry_type == ENTRY_STATUS)
-				Cmnd->result = qlogicpti_return_status(sts);
-			else
-				Cmnd->result = DID_ERROR << 16;
-
-			if(Cmnd->use_sg)
-				mmu_release_scsi_sgl((struct mmu_sglist *)
-						     Cmnd->buffer,
-						     Cmnd->use_sg - 1,
-						     qpti->qdev->my_bus);
-			else
-				mmu_release_scsi_one((__u32)((unsigned long)Cmnd->SCp.ptr),
-						     Cmnd->request_bufflen,
-						     qpti->qdev->my_bus);
-
-			qpti->cmd_count[Cmnd->target]--;
-			qregs->mbox5 = out_ptr;
-			Cmnd->scsi_done(Cmnd);
-		}
-		qpti->res_out_ptr = out_ptr;
-	}
+	spin_lock_irqsave(&io_request_lock, flags);
+	do {
+		again = 0;
+		for_each_qlogicpti(qpti)
+			again |= qlogicpti_intr_handler(qpti);
+	} while (again);
+	spin_unlock_irqrestore(&io_request_lock, flags);
 }
 
 #endif
@@ -1215,7 +1196,7 @@ static void do_qlogicpti_intr_handler(int irq, void *dev_id, struct pt_regs *reg
 	unsigned long flags;
 
 	spin_lock_irqsave(&io_request_lock, flags);
-	qlogicpti_intr_handler(irq, dev_id, regs);
+	qlogicpti_intr_handler((struct qlogicpti *)dev_id);
 	spin_unlock_irqrestore(&io_request_lock, flags);
 }
 
