@@ -394,6 +394,9 @@ int soundcore_init(void)
 #endif
 #ifdef CONFIG_SOUND_MSNDPIN
 	msnd_pinnacle_init();
+#endif
+#ifdef CONFIG_SOUND_VWSND
+	init_vwsnd();
 #endif
 	return 0;
 }
