@@ -129,10 +129,12 @@ struct inode_operations ufs_symlink_inode_operations = {
 	NULL,			/* rename */
 	ufs_readlink,		/* readlink */
 	ufs_follow_link,	/* follow_link */
+	NULL,			/* get_block */
 	NULL,			/* readpage */
 	NULL,			/* writepage */
-	NULL,			/* bmap */
+	NULL,			/* flushpage */
 	NULL,			/* truncate */
 	NULL,			/* permission */
-	NULL			/* smap */
+	NULL,			/* smap */
+	NULL			/* revalidate */
 };
