@@ -33,7 +33,7 @@ extern int	icmp_rcv(struct sk_buff *skb, unsigned short len);
 extern int	icmp_ioctl(struct sock *sk, int cmd, unsigned long arg);
 extern void	icmp_init(struct net_proto_family *ops);
 
-/* CONFIG_IP_TRANSPARENT_PROXY */
-extern int	icmp_chkaddr(struct sk_buff *skb);
+/* Move into dst.h ? */
+extern int 	xrlim_allow(struct dst_entry *dst, int timeout);
 
 #endif	/* _ICMP_H */
